# Copyright (c) 2012-2013, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.

from . import AWSHelperFn, AWSObject


Active = "Active"
Inactive = "Inactive"


class AccessKey(AWSObject):
    props = {
        'Serial': (basestring, False),
        # XXX - Is Status required? Docs say yes, examples say no
        'Status': (basestring, False),
        'UserName': (basestring, True),
    }

    def __init__(self, name, **kwargs):
        self.type = "AWS::IAM::AccessKey"
        sup = super(AccessKey, self)
        sup.__init__(name, self.type, "Properties", self.props, **kwargs)


class BasePolicy(AWSObject):
    props = {
        'Groups': (list, False),
        'PolicyDocument': (dict, True),
        'PolicyName': (basestring, True),
        'Roles': (list, False),
        'User': (list, False),
    }

    def __init__(self, name, type, propname, **kwargs):
        sup = super(BasePolicy, self)
        sup.__init__(name, type, propname, self.props, **kwargs)


class PolicyType(BasePolicy):
    # This is a top-level resource
    def __init__(self, name, **kwargs):
        self.type = "AWS::IAM::Policy"
        sup = super(PolicyType, self)
        sup.__init__(name, self.type, "Properties", **kwargs)


class Policy(BasePolicy):
    # This is for use in a list with Group (below)
    def __init__(self, name, **kwargs):
        sup = super(Policy, self)
        sup.__init__(name, None, None, **kwargs)


class Group(AWSObject):
    props = {
        'Path': (basestring, False),
        'Policies': (list, False),
    }

    def __init__(self, name, **kwargs):
        self.type = "AWS::IAM::Group"
        sup = super(Group, self)
        sup.__init__(name, self.type, "Properties", self.props, **kwargs)


class InstanceProfile(AWSObject):
    props = {
        'Path': (basestring, True),
        'Roles': (list, True),
    }

    def __init__(self, name, **kwargs):
        self.type = "AWS::IAM::InstanceProfile"
        sup = super(InstanceProfile, self)
        sup.__init__(name, self.type, "Properties", self.props, **kwargs)


class Role(AWSObject):
    props = {
        'AssumeRolePolicyDocument': (dict, True),
        'Path': (basestring, True),
        'Policies': (list, False),
    }

    def __init__(self, name, **kwargs):
        self.type = "AWS::IAM::Role"
        sup = super(Role, self)
        sup.__init__(name, self.type, "Properties", self.props, **kwargs)


class LoginProfile(AWSHelperFn):
    def __init__(self, data):
        self.data = {'Password': data}

    def JSONrepr(self):
        return self.data


class User(AWSObject):
    props = {
        'Path': (basestring, False),
        'Groups': (list, False),
        'LoginProfile': (LoginProfile, False),
        'Policies': (list, False),
    }

    def __init__(self, name, **kwargs):
        self.type = "AWS::IAM::User"
        sup = super(User, self)
        sup.__init__(name, self.type, "Properties", self.props, **kwargs)


class UserToGroupAddition(AWSObject):
    props = {
        'GroupName': (basestring, True),
        'Users': (list, True),
    }

    def __init__(self, name, **kwargs):
        self.type = "AWS::IAM::UserToGroupAddition"
        sup = super(UserToGroupAddition, self)
        sup.__init__(name, self.type, "Properties", self.props, **kwargs)



# Copyright (c) 2012-2013, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.

from . import AWSObject, AWSProperty


class Subscription(AWSProperty):
    props = {
        'Endpoint': (basestring, True),
        'Protocol': (basestring, True),
    }


class TopicPolicy(AWSObject):
    props = {
        'PolicyDocument': (dict, True),
        'Topics': (list, True),
    }

    def __init__(self, name, **kwargs):
        self.type = "AWS::SNS::TopicPolicy"
        sup = super(TopicPolicy, self)
        sup.__init__(name, self.type, "Properties", self.props, **kwargs)


class Topic(AWSObject):
    props = {
        'DisplayName': (basestring, False),
        'Subscription': (list, True),
    }

    def __init__(self, name, **kwargs):
        self.type = "AWS::SNS::Topic"
        sup = super(Topic, self)
        sup.__init__(name, self.type, "Properties", self.props, **kwargs)



# Copyright (c) 2013, Bob Van Zant <bob@veznat.com>
# All rights reserved.
#
# See LICENSE file for full license.

from . import AWSObject, AWSProperty, Tags


Private = "Private"
PublicRead = "PublicRead"
PublicReadWrite = "PublicReadWrite"
AuthenticatedRead = "AuthenticatedRead"
BucketOwnerRead = "BucketOwnerRead"
BucketOwnerFullControl = "BucketOwnerFullControl"


class WebsiteConfiguration(AWSProperty):
    props = {
        'IndexDocument': (basestring, False),
        'ErrorDocument': (basestring, False),
    }


class Bucket(AWSObject):
    props = {
        'AccessControl': (basestring, False),
        'Tags': (Tags, False),
        'WebsiteConfiguration': (WebsiteConfiguration, False)
    }

    access_control_types = [
        Private,
        PublicRead,
        PublicReadWrite,
        AuthenticatedRead,
        BucketOwnerRead,
        BucketOwnerFullControl,
    ]

    def __init__(self, name, **kwargs):
        self.type = "AWS::S3::Bucket"
        sup = super(Bucket, self)
        sup.__init__(name, self.type, "Properties", self.props, **kwargs)

        if 'AccessControl' in kwargs:
            if kwargs['AccessControl'] not in self.access_control_types:
                raise ValueError('AccessControl must be one of "%s"' % (
                    ', '.join(self.access_control_types)))


class BucketPolicy(AWSObject):
    props = {
        'Bucket': (basestring, True),
        'PolicyDocument': (dict, True),
    }

    def __init__(self, name, **kwargs):
        self.type = "AWS::S3::BucketPolicy"
        sup = super(BucketPolicy, self)
        sup.__init__(name, self.type, "Properties", self.props, **kwargs)



# Copyright (c) 2012-2013, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.

from . import AWSHelperFn, AWSObject, AWSProperty, Ref


class Tag(AWSHelperFn):
    def __init__(self, key, value):
        self.data = {'Key': key, 'Value': value}

    def JSONrepr(self):
        return self.data


class CustomGateway(AWSObject):
    props = {
        'BgpAsn': (int, True),
        'IpAddress': (basestring, True),
        'Type': (basestring, True),
    }

    def __init__(self, name, **kwargs):
        self.type = "AWS::EC2::CustomGateway"
        sup = super(CustomGateway, self)
        sup.__init__(name, self.type, "Properties", self.props, **kwargs)


class DHCPOptions(AWSObject):
    props = {
        'DomainName': (basestring, False),
        'DomainNameServers': (list, False),
        'NTPServers': (basestring, False),
        'NetbiosNameServers': (basestring, False),
        'NetbiosNodeType': (list, False),
        'Tags': (list, False),
    }

    def __init__(self, name, **kwargs):
        self.type = "AWS::EC2::DHCPOptions"
        sup = super(DHCPOptions, self)
        sup.__init__(name, self.type, "Properties", self.props, **kwargs)


class EIP(AWSObject):
    props = {
        'InstanceId': (basestring, False),
        'Domain': (basestring, False),
    }

    def __init__(self, name, **kwargs):
        self.type = "AWS::EC2::EIP"
        sup = super(EIP, self)
        sup.__init__(name, self.type, "Properties", self.props, **kwargs)


class EIPAssociation(AWSObject):
    props = {
        'AllocationId': (basestring, False),
        'EIP': (basestring, False),
        'InstanceId': (basestring, False),
        'NetworkInterfaceId': (basestring, False),
    }

    def __init__(self, name, **kwargs):
        self.type = "AWS::EC2::EIPAssociation"
        sup = super(EIPAssociation, self)
        sup.__init__(name, self.type, "Properties", self.props, **kwargs)


class Instance(AWSObject):
    props = {
        'AvailabilityZone': (basestring, False),
        'DisableApiTermination': (bool, False),
        'EbsOptimized': (bool, False),
        'IamInstanceProfile': (basestring, False),
        'ImageId': (basestring, True),
        'InstanceType': (basestring, True),
        'KernelId': (basestring, False),
        'KeyName': (basestring, False),
        'Monitoring': (basestring, False),
        'NetworkInterfaces': (list, False),
        'PlacementGroupName': (basestring, False),
        'PrivateIpAddress': (basestring, False),
        'RamdiskId': (basestring, False),
        'SecurityGroupIds': (list, False),
        'SecurityGroups': (list, False),
        'SourceDestCheck': (bool, False),
        'SubnetId': (basestring, False),
        'Tags': (list, False),
        'Tenancy': (basestring, False),
        'UserData': (basestring, False),
        'Volumes': (list, False),
    }

    def __init__(self, name, **kwargs):
        self.type = "AWS::EC2::Instance"
        sup = super(Instance, self)
        sup.__init__(name, self.type, "Properties", self.props, **kwargs)


class MountPoint(AWSProperty):
    props = {
        'Device': (basestring, True),
        'VolumeId': (basestring, True),
    }


class InternetGateway(AWSObject):
    props = {
        'Tags': (list, False),
    }

    def __init__(self, name, **kwargs):
        self.type = "AWS::EC2::InternetGateway"
        sup = super(InternetGateway, self)
        sup.__init__(name, self.type, "Properties", self.props, **kwargs)


class NetworkAcl(AWSObject):
    props = {
        'Tags': (list, False),
        'VpcId': (basestring, True),
    }

    def __init__(self, name, **kwargs):
        self.type = "AWS::EC2::NetworkAcl"
        sup = super(NetworkAcl, self)
        sup.__init__(name, self.type, "Properties", self.props, **kwargs)


class ICMP(AWSProperty):
    props = {
        'Code': (int, False),
        'Type': (int, False),
    }


class PortRange(AWSProperty):
    props = {
        'From': (int, False),
        'To': (int, False),
    }


class NetworkAclEntry(AWSObject):
    props = {
        'NetworkAclId': (basestring, True),
        'RuleNumber': (int, True),
        'Protocol': (int, True),
        'RuleAction': (basestring, True),
        'Egress': (basestring, True),
        'CidrBlock': (basestring, True),
        'Icmp': (ICMP, True),
        'PortRange': (PortRange, True),
    }

    def __init__(self, name, **kwargs):
        self.type = "AWS::EC2::NetworkAclEntry"
        sup = super(NetworkAclEntry, self)
        sup.__init__(name, self.type, "Properties", self.props, **kwargs)


class NetworkInterface(AWSObject):
    props = {
        'Description': (basestring, False),
        'GroupSet': (list, False),
        'PrivateIpAddress': (basestring, False),
        'SourceDestCheck': (bool, False),
        'SubnetId': (basestring, True),
        'Tags': (list, False),
    }

    def __init__(self, name, **kwargs):
        self.type = "AWS::EC2::NetworkInterface"
        sup = super(NetworkInterface, self)
        sup.__init__(name, self.type, "Properties", self.props, **kwargs)


class Route(AWSObject):
    props = {
        'DestinationCidrBlock': (basestring, True),
        'GatewayId': (basestring, False),
        'InstanceId': (basestring, False),
        'NetworkInterfaceId': (basestring, False),
        'RouteTableId': (basestring, True),
    }

    def __init__(self, name, **kwargs):
        self.type = "AWS::EC2::Route"
        sup = super(Route, self)
        sup.__init__(name, self.type, "Properties", self.props, **kwargs)


class RouteTable(AWSObject):
    props = {
        'Tags': (list, False),
        'VpcId': (basestring, True),
    }

    def __init__(self, name, **kwargs):
        self.type = "AWS::EC2::RouteTable"
        sup = super(RouteTable, self)
        sup.__init__(name, self.type, "Properties", self.props, **kwargs)


class SecurityGroupEgress(AWSObject):
    props = {
        'GroupId': (basestring, True),
        'IpProtocol': (basestring, True),
        'CidrIp': (basestring, False),
        'DestinationSecurityGroupId': (basestring, False),
        'FromPort': (basestring, True),
        'ToPort': (basestring, True),
    }

    def __init__(self, name, **kwargs):
        self.type = "AWS::EC2::SecurityGroupEgress"
        sup = super(SecurityGroupEgress, self)
        sup.__init__(name, self.type, "Properties", self.props, **kwargs)


class SecurityGroupIngress(AWSObject):
    props = {
        'GroupName': (basestring, False),
        'GroupId': (basestring, False),
        'IpProtocol': (basestring, True),
        'CidrIp': (basestring, False),
        'SourceSecurityGroupName': (basestring, False),
        'SourceSecurityGroupId': (basestring, False),
        'SourceSecurityGroupOwnerId': (basestring, False),
        'FromPort': (basestring, False),
        'ToPort': (basestring, False),
    }

    def __init__(self, name, **kwargs):
        self.type = "AWS::EC2::SecurityGroupIngress"
        sup = super(SecurityGroupIngress, self)
        sup.__init__(name, self.type, "Properties", self.props, **kwargs)


class SecurityGroupRule(AWSProperty):
    props = {
        'IpProtocol': (basestring, True),
        'CidrIp': (basestring, False),
        'SourceSecurityGroupName': (basestring, False),
        'SourceSecurityGroupId': (basestring, False),
        'SourceSecurityGroupOwnerId': (basestring, False),
        'FromPort': (basestring, True),
        'ToPort': (basestring, True),
    }


class SecurityGroup(AWSObject):
    props = {
        'GroupDescription': (basestring, True),
        'SecurityGroupEgress': (list, False),
        'SecurityGroupIngress': (list, False),
        'VpcId': (basestring, False),
    }

    def __init__(self, name, **kwargs):
        self.type = "AWS::EC2::SecurityGroup"
        sup = super(SecurityGroup, self)
        sup.__init__(name, self.type, "Properties", self.props, **kwargs)


class Subnet(AWSObject):
    props = {
        'AvailabilityZone': (basestring, False),
        'CidrBlock': (basestring, True),
        'Tags': (list, False),
        'VpcId': (Ref, True),
    }

    def __init__(self, name, **kwargs):
        self.type = "AWS::EC2::Subnet"
        sup = super(Subnet, self)
        sup.__init__(name, self.type, "Properties", self.props, **kwargs)


class SubnetNetworkAclAssociation(AWSObject):
    props = {
        'SubnetId': (basestring, True),
        'NetworkAclId': (basestring, True),
    }

    def __init__(self, name, **kwargs):
        self.type = "AWS::EC2::SubnetNetworkAclAssociation"
        sup = super(SubnetNetworkAclAssociation, self)
        sup.__init__(name, self.type, "Properties", self.props, **kwargs)


class SubnetRouteTableAssociation(AWSObject):
    props = {
        'RouteTableId': (basestring, True),
        'SubnetId': (basestring, True),
    }

    def __init__(self, name, **kwargs):
        self.type = "AWS::EC2::SubnetRouteTableAssociation"
        sup = super(SubnetRouteTableAssociation, self)
        sup.__init__(name, self.type, "Properties", self.props, **kwargs)


class Volume(AWSObject):
    props = {
        'AvailabilityZone': (basestring, True),
        'Iops': (int, False),
        'Size': (basestring, False),
        'SnapshotId': (basestring, False),
        'Tags': (list, False),
        'VolumeType': (basestring, False),
    }

    def __init__(self, name, **kwargs):
        self.type = "AWS::EC2::Volume"
        sup = super(Volume, self)
        sup.__init__(name, self.type, "Properties", self.props, **kwargs)


class VolumeAttachment(AWSObject):
    props = {
        'Device': (basestring, True),
        'InstanceId': (basestring, True),
        'VolumeId': (basestring, True),
    }

    def __init__(self, name, **kwargs):
        self.type = "AWS::EC2::VolumeAttachment"
        sup = super(VolumeAttachment, self)
        sup.__init__(name, self.type, "Properties", self.props, **kwargs)


class VPC(AWSObject):
    props = {
        'CidrBlock': (basestring, True),
        'InstanceTenancy': (basestring, False),
        'Tags': (list, False),
    }

    def __init__(self, name, **kwargs):
        self.type = "AWS::EC2::VPC"
        sup = super(VPC, self)
        sup.__init__(name, self.type, "Properties", self.props, **kwargs)


class VPCDHCPOptionsAssociation(AWSObject):
    props = {
        'VpcId': (basestring, True),
        'DhcpOptionsId': (basestring, True),
    }

    def __init__(self, name, **kwargs):
        self.type = "AWS::EC2::VPCDHCPOptionsAssociation"
        sup = super(VPCDHCPOptionsAssociation, self)
        sup.__init__(name, self.type, "Properties", self.props, **kwargs)


class VPCGatewayAttachment(AWSObject):
    props = {
        'VpcId': (basestring, True),
        'InternetGatewayId': (basestring, False),
        'VpnGatewayId': (basestring, False),
    }

    def __init__(self, name, **kwargs):
        self.type = "AWS::EC2::VPCGatewayAttachment"
        sup = super(VPCGatewayAttachment, self)
        sup.__init__(name, self.type, "Properties", self.props, **kwargs)


class VPNConnection(AWSObject):
    props = {
        'Type': (basestring, True),
        'CustomerGatewayId': (basestring, True),
        'VpnGatewayId': (basestring, True),
    }

    def __init__(self, name, **kwargs):
        self.type = "AWS::EC2::VPNConnection"
        sup = super(VPNConnection, self)
        sup.__init__(name, self.type, "Properties", self.props, **kwargs)


class VPNGateway(AWSObject):
    props = {
        'Type': (basestring, True),
        'Tags': (list, False),
    }

    def __init__(self, name, **kwargs):
        self.type = "AWS::EC2::VPNGateway"
        sup = super(VPNGateway, self)
        sup.__init__(name, self.type, "Properties", self.props, **kwargs)



# Copyright (c) 2013, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.

from . import AWSObject, Ref


class Stack(AWSObject):
    props = {
        'TemplateURL': (basestring, True),
        'TimeoutInMinutes': (basestring, False),
        'Parameters': (dict, False),
    }

    def __init__(self, name, **kwargs):
        self.type = "AWS::CloudFormation::Stack"
        sup = super(Stack, self)
        sup.__init__(name, self.type, "Properties", self.props, **kwargs)


class WaitCondition(AWSObject):
    props = {
        'Handle': (Ref, True),
        'Timeout': (basestring, True),
        'Count': (basestring, False),
    }

    def __init__(self, name, **kwargs):
        self.type = "AWS::CloudFormation::WaitCondition"
        sup = super(WaitCondition, self)
        sup.__init__(name, self.type, "Properties", self.props, **kwargs)


class WaitConditionHandle(AWSObject):
    props = {}

    def __init__(self, name, **kwargs):
        self.type = "AWS::CloudFormation::WaitConditionHandle"
        sup = super(WaitConditionHandle, self)
        sup.__init__(name, self.type, "Properties", self.props, **kwargs)



# Copyright (c) 2012-2013, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.

from . import AWSHelperFn, AWSObject, AWSProperty


class ForwardedValues(AWSHelperFn):
    def __init__(self, querystring):
        if not isinstance(querystring, bool):
            raise TypeError
        self.data = {
            'QueryString': querystring,
        }

    def JSONrepr(self):
        return self.data


class DefaultCacheBehavior(AWSProperty):
    props = {
        'TargetOriginId': (basestring, True),
        'ForwardedValues': (ForwardedValues, False),
        'TrustedSigners': (list, False),
        'ViewerProtocolPolicy': (basestring, False),
        'MinTTL': (basestring, False),
    }


class S3Origin(AWSHelperFn):
    def __init__(self, originaccessidentity):
        if not isinstance(originaccessidentity, basestring):
            raise TypeError
        self.data = {
            'OriginAccessIdentity': originaccessidentity,
        }

    def JSONrepr(self):
        return self.data


class CustomOrigin(AWSProperty):
    props = {
        'HTTPPort': (basestring, False),
        'HTTPSPort': (basestring, False),
        'OriginProtocolPolicy': (basestring, True),
    }


class Origin(AWSProperty):
    props = {
        'DomainName': (basestring, True),
        'Id': (basestring, True),
        'S3OriginConfig': (S3Origin, False),
        'CustomOriginConfig': (CustomOrigin, False),
    }


class Logging(AWSProperty):
    props = {
        'Bucket': (basestring, True),
        'Prefix': (basestring, False),
    }


class DistributionConfig(AWSProperty):
    props = {
        'Aliases': (list, False),
        'CacheBehaviors': (list, False),
        'Comment': (basestring, False),
        'DefaultCacheBehavior': (DefaultCacheBehavior, False),
        'DefaultRootObject': (basestring, False),
        'Enabled': (bool, True),
        'Logging': (Logging, False),
        'Origins': (list, True),
    }


class Distribution(AWSObject):
    props = {
        'DistributionConfig': (DistributionConfig, True),
    }

    def __init__(self, name, **kwargs):
        self.type = "AWS::CloudFront::Distribution"
        sup = super(Distribution, self)
        sup.__init__(name, self.type, "Properties", self.props, **kwargs)



# Copyright (c) 2013, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.

from . import AWSObject, AWSProperty


class SourceBundle(AWSProperty):
    props = {
        'S3Bucket': (basestring, True),
        'S3Key': (basestring, True),
    }


class ApplicationVersion(AWSProperty):
    props = {
        'Description': (basestring, False),
        'SourceBundle': (SourceBundle, False),
        'VersionLabel': (basestring, True),
    }


class OptionSettings(AWSProperty):
    props = {
        'Namespace': (basestring, True),
        'OptionName': (basestring, True),
        'Value': (basestring, True),
    }


class ConfigurationTemplate(AWSProperty):
    props = {
        'TemplateName': (basestring, True),
        'Description': (basestring, False),
        'OptionSettings': (list, False),
        'SolutionStackName': (basestring, False),
    }


class Application(AWSObject):
    props = {
        'ApplicationVersions': (list, True),
        'ConfigurationTemplates': (list, False),
        'Description': (basestring, False),
    }

    def __init__(self, name, **kwargs):
        self.type = "AWS::ElasticBeanstalk::Application"
        sup = super(Application, self)
        sup.__init__(name, self.type, "Properties", self.props, **kwargs)


class Environment(AWSObject):
    props = {
        'ApplicationName': (basestring, True),
        'CNAMEPrefix': (basestring, False),
        'Description': (basestring, False),
        'OptionSettings': (list, False),
        'OptionsToRemove': (list, False),
        'SolutionStackName': (basestring, False),
        'TemplateName': (basestring, False),
        'VersionLabel': (basestring, False),
    }

    def __init__(self, name, **kwargs):
        self.type = "AWS::ElasticBeanstalk::Environment"
        sup = super(Environment, self)
        sup.__init__(name, self.type, "Properties", self.props, **kwargs)



# Copyright (c) 2012-2013, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.

from . import AWSObject


class Domain(AWSObject):
    props = {
        'Description': (basestring, False),
    }

    def __init__(self, name, **kwargs):
        self.type = "AWS::SDB::Domain"
        sup = super(Domain, self)
        sup.__init__(name, self.type, "Properties", self.props, **kwargs)



# Copyright (c) 2013, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.

from . import AWSObject, AWSProperty


class DBInstance(AWSObject):
    props = {
        'AllocatedStorage': (basestring, True),
        'AutoMinorVersionUpgrade': (bool, False),
        'AvailabilityZone': (basestring, False),
        'BackupRetentionPeriod': (basestring, False),
        'DBInstanceClass': (basestring, True),
        'DBName': (basestring, False),
        'DBParameterGroupName': (basestring, False),
        'DBSecurityGroups': (list, False),
        'DBSnapshotIdentifier': (basestring, False),
        'DBSubnetGroupName': (basestring, False),
        'Engine': (basestring, True),
        'EngineVersion': (basestring, False),
        'Iops': (int, False),
        'LicenseModel': (basestring, False),
        'MasterUsername': (basestring, True),
        'MasterUserPassword': (basestring, True),
        'MultiAZ': (basestring, False),
        'Port': (basestring, False),
        'PreferredBackupWindow': (basestring, False),
        'PreferredMaintenanceWindow': (basestring, False),
        'Tags': (list, False),
    }

    def __init__(self, name, **kwargs):
        self.type = "AWS::RDS::DBInstance"
        sup = super(DBInstance, self)
        sup.__init__(name, self.type, "Properties", self.props, **kwargs)


class DBParameterGroup(AWSObject):
    props = {
        'Description': (basestring, False),
        'Family': (basestring, False),
        'Parameters': (dict, False),
    }

    def __init__(self, name, **kwargs):
        self.type = "AWS::RDS::DBParameterGroup"
        sup = super(DBParameterGroup, self)
        sup.__init__(name, self.type, "Properties", self.props, **kwargs)


class DBSubnetGroup(AWSObject):
    props = {
        'DBSubnetGroupDescription': (basestring, True),
        'SubnetIds': (list, True),
    }

    def __init__(self, name, **kwargs):
        self.type = "AWS::RDS::DBSubnetGroup"
        sup = super(DBSubnetGroup, self)
        sup.__init__(name, self.type, "Properties", self.props, **kwargs)


class RDSSecurityGroup(AWSProperty):
    props = {
        'CIDRIP': (basestring, False),
        'EC2SecurityGroupId': (basestring, False),
        'EC2SecurityGroupName': (basestring, False),
        'EC2SecurityGroupOwnerId': (basestring, False),
    }


class DBSecurityGroup(AWSObject):
    props = {
        'EC2VpcId': (basestring, False),
        'DBSecurityGroupIngress': (list, True),
        'GroupDescription': (basestring, True),
    }

    def __init__(self, name, **kwargs):
        self.type = "AWS::RDS::DBSecurityGroup"
        sup = super(DBSecurityGroup, self)
        sup.__init__(name, self.type, "Properties", self.props, **kwargs)


class DBSecurityGroupIngress(AWSObject):
    props = {
        'CIDRIP': (basestring, False),
        'DBSecurityGroupName': (basestring, True),
        'EC2SecurityGroupId': (basestring, True),
        'EC2SecurityGroupName': (basestring, True),
        'EC2SecurityGroupOwnerId': (basestring, True),
    }

    def __init__(self, name, **kwargs):
        self.type = "AWS::RDS::DBSecurityGroupIngress"
        sup = super(DBSecurityGroupIngress, self)
        sup.__init__(name, self.type, "Properties", self.props, **kwargs)



# Copyright (c) 2013, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.

from . import AWSObject, AWSProperty


class Alarm(AWSObject):
    props = {
        'ActionsEnabled': (basestring, False),
        'AlarmActions': (list, False),
        'AlarmDescription': (basestring, False),
        'ComparisonOperator': (basestring, True),
        'Dimensions': (list, False),
        'EvaluationPeriods': (basestring, True),
        'InsufficientDataActions': (list, False),
        'MetricName': (basestring, True),
        'Namespace': (basestring, True),
        'OKActions': (list, False),
        'Period': (basestring, True),
        'Statistic': (basestring, True),
        'Threshold': (basestring, True),
        'Unit': (basestring, False),
    }

    def __init__(self, name, **kwargs):
        self.type = "AWS::CloudWatch::Alarm"
        sup = super(Alarm, self)
        sup.__init__(name, self.type, "Properties", self.props, **kwargs)


class MetricDimension(AWSProperty):
    props = {
        'Name': (basestring, True),
        'Value': (basestring, True),
    }



# Copyright (c) 2012-2013, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.

from . import AWSHelperFn, AWSObject


class AliasTarget(AWSHelperFn):
    def __init__(self, hostedzoneid, dnsname):
        self.data = {
            'HostedZoneId': hostedzoneid,
            'DNSName': dnsname,
        }

    def JSONrepr(self):
        return self.data


class BaseRecordSet(AWSObject):
    props = {
        'AliasTarget': (AliasTarget, False),
        'Comment': (basestring, False),
        'HostedZoneId': (basestring, False),
        'HostedZoneName': (basestring, False),
        'Name': (basestring, True),
        'Region': (basestring, False),
        'ResourceRecords': (list, False),
        'SetIdentifier': (basestring, False),
        'TTL': (basestring, False),
        'Type': (basestring, True),
        'Weight': (basestring, False),
    }

    def __init__(self, name, type, propname, **kwargs):
        sup = super(BaseRecordSet, self)
        sup.__init__(name, type, propname, self.props, **kwargs)


class RecordSetType(BaseRecordSet):
    # This is a top-level resource
    def __init__(self, name, **kwargs):
        self.type = "AWS::Route53::RecordSet"
        sup = super(RecordSetType, self)
        sup.__init__(name, self.type, "Properties", **kwargs)


class RecordSet(BaseRecordSet):
    # This is for use in a list with RecordSetGroup (below)
    def __init__(self, **kwargs):
        sup = super(RecordSet, self)
        sup.__init__(None, None, None, **kwargs)


class RecordSetGroup(AWSObject):
    props = {
        'HostedZoneId': (basestring, False),
        'HostedZoneName': (basestring, False),
        'RecordSets': (list, False),
        'Comment': (basestring, False),
    }

    def __init__(self, name, **kwargs):
        self.type = "AWS::Route53::RecordSetGroup"
        sup = super(RecordSetGroup, self)
        sup.__init__(name, self.type, "Properties", self.props, **kwargs)



# Copyright (c) 2013, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.

from . import AWSObject


class CacheCluster(AWSObject):
    props = {
        'AutoMinorVersionUpgrade': (bool, False),
        'CacheNodeType': (basestring, True),
        'CacheParameterGroupName': (basestring, False),
        'CacheSecurityGroupNames': (list, True),
        'Engine': (basestring, True),
        'EngineVersion': (basestring, False),
        'NotificationTopicArn': (basestring, False),
        'NumCacheNodes': (basestring, False),
        'Port': (int, False),
        'PreferredAvailabilityZone': (basestring, False),
        'PreferredMaintenanceWindow': (basestring, False),
    }

    def __init__(self, name, **kwargs):
        self.type = "AWS::ElastiCache::CacheCluster"
        sup = super(CacheCluster, self)
        sup.__init__(name, self.type, "Properties", self.props, **kwargs)


class ParameterGroup(AWSObject):
    props = {
        'CacheParameterGroupFamily': (basestring, True),
        'Description': (basestring, True),
        'Properties': (dict, True),
    }

    def __init__(self, name, **kwargs):
        self.type = "AWS::ElastiCache::ParameterGroup"
        sup = super(ParameterGroup, self)
        sup.__init__(name, self.type, "Properties", self.props, **kwargs)


class SecurityGroup(AWSObject):
    props = {
        'Description': (basestring, True),
    }

    def __init__(self, name, **kwargs):
        self.type = "AWS::ElastiCache::SecurityGroup"
        sup = super(SecurityGroup, self)
        sup.__init__(name, self.type, "Properties", self.props, **kwargs)


class SecurityGroupIngress(AWSObject):
    props = {
        'CacheSecurityGroupName': (basestring, True),
        'EC2SecurityGroupName': (basestring, True),
        'EC2SecurityGroupOwnerId': (basestring, False),
    }

    def __init__(self, name, **kwargs):
        self.type = "AWS::ElastiCache::SecurityGroupIngress"
        sup = super(SecurityGroupIngress, self)
        sup.__init__(name, self.type, "Properties", self.props, **kwargs)



# Copyright (c) 2012-2013, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.


import json
import re


__version__ = "0.2.0"

# constants for DeletionPolicy
Delete = 'Delete'
Retain = 'Retain'
Snapshot = 'Snapshot'

valid_names = re.compile(r'^[a-zA-Z0-9]+$')


class AWSObject(object):
    def __init__(self, name, type=None, dictname=None, props={}, **kwargs):
        self.name = name
        self.type = type
        self.props = props
        # Cache the keys for validity checks
        self.propnames = props.keys()
        self.attributes = ['DependsOn', 'DeletionPolicy',
                           'Metadata', 'UpdatePolicy']

        # unset/None is also legal
        if name and not valid_names.match(name):
            raise ValueError('Name not alphanumeric')

        # Create the list of properties set on this object by the user
        self.properties = {}
        if dictname:
            self.resource = {
                dictname: self.properties,
            }
        else:
            self.resource = self.properties
        if self.type:
            self.resource['Type'] = self.type
        self.__initialized = True

        # Now that it is initialized, populate it with the kwargs
        for k, v in kwargs.items():
            # Special case Resource Attributes
            if k in self.attributes:
                self.resource[k] = v
            else:
                self.__setattr__(k, v)

    def __getattr__(self, name):
        try:
            return self.properties.__getitem__(name)
        except KeyError:
            raise AttributeError(name)

    def __setattr__(self, name, value):
        if '_AWSObject__initialized' not in self.__dict__:
            return dict.__setattr__(self, name, value)
        elif name in self.propnames:
            # Check the type of the object and compare against what
            # we were expecting. Special case AWS helper functions.
            expected_type = self.props[name][0]
            if isinstance(value, expected_type) or \
                    isinstance(value, AWSHelperFn):
                return self.properties.__setitem__(name, value)
            else:
                raise TypeError('%s is %s, expected %s' %
                                (name, type(value), expected_type))
        raise AttributeError("%s object does not support attribute %s" %
                            (self.type, name))

    def JSONrepr(self):
        for k, v in self.props.items():
            if v[1] and k not in self.properties:
                raise ValueError("Resource %s required in type %s" %
                                 (k, self.type))
        # If no other properties are set, only return the Type.
        # Mainly used to not have an empty "Properties".
        if self.properties:
            return self.resource
        else:
            return {'Type': self.type}


class AWSProperty(AWSObject):
    """
    Used for CloudFormation Resource Property objects
    http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/
    aws-product-property-reference.html
    """

    def __init__(self, **kwargs):
        sup = super(AWSProperty, self)
        sup.__init__(None, props=self.props, **kwargs)


class UpdatePolicy(AWSObject):
    props = {
        'MaxBatchSize': (basestring, False),
        'MinInstancesInService': (basestring, False),
        'PauseTime': (basestring, False),
    }

    def __init__(self, name, **kwargs):
        sup = super(UpdatePolicy, self)
        sup.__init__(None, None, name, self.props, **kwargs)


class AWSHelperFn(object):
    def getdata(self, data):
        if isinstance(data, AWSObject):
            return data.name
        else:
            return data


class Base64(AWSHelperFn):
    def __init__(self, data):
        self.data = {'Fn::Base64': data}

    def JSONrepr(self):
        return self.data


class FindInMap(AWSHelperFn):
    def __init__(self, map, key, value):
        self.data = {'Fn::FindInMap': [self.getdata(map), key, value]}

    def JSONrepr(self):
        return self.data


class GetAtt(AWSHelperFn):
    def __init__(self, logicalName, attrName):
        self.data = {'Fn::GetAtt': [self.getdata(logicalName), attrName]}

    def JSONrepr(self):
        return self.data


class GetAZs(AWSHelperFn):
    def __init__(self, region):
        self.data = {'Fn::GetAZs': region}

    def JSONrepr(self):
        return self.data


class Join(AWSHelperFn):
    def __init__(self, delimiter, values):
        self.data = {'Fn::Join': [delimiter, values]}

    def JSONrepr(self):
        return self.data


class Name(AWSHelperFn):
    def __init__(self, data):
        self.data = self.getdata(data)

    def JSONrepr(self):
        return self.data


class Select(AWSHelperFn):
    def __init__(self, indx, objects):
        self.data = {'Fn::Select': [indx, objects]}

    def JSONrepr(self):
        return self.data


class Ref(AWSHelperFn):
    def __init__(self, data):
        self.data = {'Ref': self.getdata(data)}

    def JSONrepr(self):
        return self.data


class awsencode(json.JSONEncoder):
    def default(self, obj):
        if hasattr(obj, 'JSONrepr'):
            return obj.JSONrepr()
        return json.JSONEncoder.default(self, obj)


class Tags(object):
    def __init__(self, **kwargs):
        self.tags = []
        for k, v in kwargs.iteritems():
            self.tags.append({
                'Key': k,
                'Value': v,
            })

    def JSONrepr(self):
        return self.tags


class Template(object):
    props = {
        'AWSTemplateFormatVersion': (basestring, False),
        'Description': (basestring, False),
        'Parameters': (dict, False),
        'Mappings': (dict, False),
        'Resources': (dict, False),
        'Outputs': (dict, False),
    }

    def __init__(self):
        self.description = None
        self.mappings = {}
        self.outputs = {}
        self.parameters = {}
        self.resources = {}
        self.version = None

    def add_description(self, description):
        self.description = description

    def add_output(self, output):
        if isinstance(output, list):
            for o in output:
                self.outputs[o.name] = o
        else:
            self.outputs[output.name] = output
        return output

    def add_mapping(self, name, mapping):
        self.mappings[name] = mapping

    def add_parameter(self, parameter):
        if isinstance(parameter, list):
            for p in parameter:
                self.parameters[p.name] = p
        else:
            self.parameters[parameter.name] = parameter
        return parameter

    def add_resource(self, resource):
        if isinstance(resource, list):
            for r in resource:
                self.resources[r.name] = r
        else:
            self.resources[resource.name] = resource
        return resource

    def add_version(self, version=None):
        if version:
            self.version = version
        else:
            self.version = "2010-09-09"

    def to_json(self, indent=4, sort_keys=True):
        t = {}
        if self.description:
            t['Description'] = self.description
        if self.mappings:
            t['Mappings'] = self.mappings
        if self.outputs:
            t['Outputs'] = self.outputs
        if self.parameters:
            t['Parameters'] = self.parameters
        if self.version:
            t['AWSTemplateFormatVersion'] = self.version
        t['Resources'] = self.resources

        return json.dumps(t, cls=awsencode, indent=indent, sort_keys=sort_keys)

    def JSONrepr(self):
        return [self.parameters, self.mappings, self.resources]


class Output(AWSObject):
    props = {
        'Description': (basestring, False),
        'Value': (basestring, True),
    }

    def __init__(self, name, **kwargs):
        sup = super(Output, self)
        sup.__init__(name, None, props=self.props, **kwargs)


class Parameter(AWSObject):
    props = {
        'Type': (basestring, True),
        'Default': (basestring, False),
        'NoEcho': (bool, False),
        'AllowedValues': (list, False),
        'AllowedPattern': (basestring, False),
        'MaxLength': (basestring, False),
        'MinLength': (basestring, False),
        'MaxValue': (basestring, False),
        'MinValue': (basestring, False),
        'Description': (basestring, False),
        'ConstraintDescription': (basestring, False),
    }

    def __init__(self, name, **kwargs):
        self.type = None
        sup = super(Parameter, self)
        sup.__init__(name, props=self.props, **kwargs)



from setuptools import setup

setup(
    name='troposphere',
    version='0.2.0',
    description="AWS CloudFormation creation library",
    author="Mark Peek",
    author_email="mark@peek.org",
    url="https://github.com/cloudtools/troposphere",
    license="New BSD license",
    packages=['troposphere'],
    use_2to3=True,
)
